<?php
declare(strict_types=1);

namespace Inti\Image\Convert;

use finfo;
use RuntimeException;
use function fopen;
use function fwrite;
use function imagecreatefromstring;
use function imagegif;
use function imagejpeg;
use function imagepng;
use function imagewebp;
use function ob_end_clean;
use function ob_get_contents;
use function ob_start;
use function rewind;
use function sprintf;
use function stream_get_contents;

/**
 * Class GdConvert
 *
 * @package Inti\Image\Convert
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-image
 */
final class GdConvert implements ConvertInterface
{
    /**
     * @param string $contents
     * @param string $format
     * @return string
     */
    public function convert(string $contents, string $format): string
    {
        $finfo = new finfo(FILEINFO_MIME_TYPE);
        $current = $finfo->buffer($contents);

        if ($current === $format) {
            return $contents;
        }

        $image = imagecreatefromstring($contents);

        ob_start();

        switch ($format) {
            case 'image/gif':
                imagegif($image);
                break;
            case 'image/jpeg':
                imagejpeg($image, null, 100);
                break;
            case 'image/png':
                imagepng($image);
                break;
            case 'image/webp':
                imagewebp($image);
                break;
            default:
                throw new RuntimeException(sprintf(
                    '%s: Cannot convert image to %s.', __METHOD__, $format
                ));
        }

        $converted = ob_get_contents();
        ob_end_clean();

        return $converted;
    }

    /**
     * @param string $path
     * @param string $format
     * @return string
     */
    public function convertFile(string $path, string $format): string
    {
        $stream = fopen($path, 'r');
        $converted = $this->convertStream($stream, $format);
        return stream_get_contents($converted);
    }

    /**
     * @param resource $stream
     * @param string $format
     * @return resource|void
     */
    public function convertStream($stream, string $format)
    {
        $contents = stream_get_contents($stream);
        $converted = fopen('php://memory', 'r+b');

        fwrite($converted, $this->convert($contents, $format));
        rewind($converted);

        return $converted;
    }
}
