<?php
declare(strict_types=1);

namespace Inti\Image\Optimize;

use Spatie\ImageOptimizer\OptimizerChainFactory;
use function fopen;
use function fwrite;
use function rewind;
use function stream_get_contents;
use function stream_get_meta_data;
use function tmpfile;

/**
 * Class DefaultOptimize
 *
 * @package Inti\Image\Optimize
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-image
 */
final class DefaultOptimize implements OptimizeInterface
{
    /**
     * @param string $contents
     * @return string
     */
    public function optimize(string $contents): string
    {
        $stream = tmpfile();
        fwrite($stream, $contents);
        rewind($stream);

        $optimized = $this->optimizeStream($stream);
        $contents = stream_get_contents($optimized);

        fclose($stream);
        fclose($optimized);
        return $contents;
    }

    public function optimizeFile(string $path): void
    {
        $optimizer = OptimizerChainFactory::create();
        $optimizer->optimize($path);
    }

    /**
     * @param resource $stream
     * @return false|resource
     */
    public function optimizeStream($stream)
    {
        $meta = stream_get_meta_data($stream);
        $path = $meta['uri'];

        $this->optimizeFile($path);

        return fopen($path, 'r');
    }
}
