<?php
declare(strict_types=1);

namespace Inti\Image\Resize;

use Imagick;
use ImagickException;

/**
 * Class ImagickResize
 *
 * @package Inti\Image\Resize
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-image
 */
final class ImagickResize implements ResizeInterface
{
    /**
     * @param string $contents
     * @param int|null $width
     * @param int|null $height
     * @param bool $crop
     * @return string
     * @throws ImagickException
     */
    public function resize(string $contents, int $width = null, int $height = null, bool $crop = false): string
    {
        $imagick = new Imagick();
        $imagick->readImageBlob($contents);

        $imageWidth = $imagick->getImageWidth();
        $imageHeight = $imagick->getImageHeight();
        $imageRatio = $imageWidth / $imageHeight;
        $bestfit = false;

        if ($width === null || $height === null) {
            if ($width) {
                $newWidth = $width;
                $newHeight = $width / $imageRatio;
            } else {
                $newWidth = $height * $imageRatio;
                $newHeight = $height;
            }
        } else  {
            $bestfit = true;
            $newWidth = $width;
            $newHeight = $height;
        }

        if ($crop) {
            $bestfit = false;
        }

        $imagick->resizeImage(intval($newWidth), intval($newHeight), Imagick::FILTER_UNDEFINED, 1, $bestfit);
        return $imagick->getImageBlob();
    }
}
